#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_ReceiverParmObj    parm;
    TSYNC_ReceiverParmUseObj cmd = {0};


    /* If invalid number of arguments... */
    if (argc < 5)
    {
        printf(" Usage: GR_SetParm <device index> <index> <type> <args>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Send Receiver Specific Parameter message */
    index      = atoi(argv[2]);

    /* Limited support is implemented at this time for ONLY COM1 usage */
    cmd.type   = atoi(argv[3]); 
    cmd.use    = atoi(argv[4]);
    parm.len   = sizeof(TSYNC_ReceiverParmUseObj);

    printf("\n");
    printf(" GR (%d) Set Parameter Msg Sent:\n", (index));
    printf("  ------------------------\n");

    switch (cmd.type)
    {
        case GL_PARM_GSSIP_COM1_USE:
            {    
                switch (cmd.use)
                {
                    case GL_COM1_HS_OUT:
                        printf("COM1 used for HotStart Output (%d)", cmd.use);
                        break;
                    case GL_COM1_HS_IN:
                        printf("COM1 used for HotStart Input (%d)", cmd.use);
                        break;
                    case GL_COM1_IDLE:
                        printf("COM1 Idle - not used (%d)", cmd.use);
                        break;
                    default:
                        printf("Unknown COM1 use (%d)", cmd.use);
                        break;
                }
            }
            break;
        default:
            break;
    }

    printf("\n");

    // Convert from the Hosts (ETX) byte order to Big Endian (KTS Byte Order)
    cmd.type  = htonl(cmd.type);
    cmd.use   = htonl(cmd.use);
    memcpy(parm.parm, &cmd, parm.len); 

    // Send Send Custom Message transaction
    err = TSYNC_GR_setParameter(hnd, index, &parm);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
